#
# Copyright (C) 2021 Satoru SATOH <satoru.satoh@gmail.com>
# SPDX-License-Identifier: MIT
#
"""Some common constants, utility functions and so on.
"""
import typing


ResultT = typing.Tuple[bool, typing.Union[str, typing.List[str]]]

# vim:sw=4:ts=4:et:
