#
# Copyright (C) 2021 Satoru SATOH <satoru.satoh@gmail.com>
# SPDX-License-Identifier: MIT
#
r"""Common data types for anyconfig.query.
"""
import typing


MaybeJexp = typing.Optional[typing.Union[str, bool]]

# vim:sw=4:ts=4:et:
